@@ -17,6 +17,6 @@
 
 from __future__ import annotations
 
-__version__ = '2.17.1'
+__version__ = '2.19.0'
 __author__ = 'Ansible, Inc.'
-__codename__ = "Gallows Pole"
+__codename__ = "What Is and What Should Never Be"
