@@ -2,8 +2,7 @@
 # Copyright: (c) 2012-17, Ansible Project
 # GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
 
-from __future__ import (absolute_import, division, print_function)
-__metaclass__ = type
+from __future__ import annotations
 
 DOCUMENTATION = """
     name: template
@@ -18,9 +17,13 @@ DOCUMENTATION = """
       convert_data:
         type: bool
         description:
-            - Whether to convert YAML into data. If False, strings that are YAML will be left untouched.
-            - Mutually exclusive with the jinja2_native option.
+            - Whether to convert YAML into data. If V(False), strings that are YAML will be left untouched.
+            - Mutually exclusive with the O(jinja2_native) option.
         default: true
+        deprecated:
+          why: This option is no longer used in the Ansible Core code base.
+          version: "2.23"
+          alternatives: Jinja2 native mode is now the default and only option, which is mutually exclusive with this option.
       variable_start_string:
         description: The string marking the beginning of a print statement.
         default: '{{'
@@ -34,13 +37,16 @@ DOCUMENTATION = """
       jinja2_native:
         description:
             - Controls whether to use Jinja2 native types.
-            - It is off by default even if global jinja2_native is True.
-            - Has no effect if global jinja2_native is False.
+            - It is off by default even if global O(jinja2_native) is V(True).
+            - Has no effect if global O(jinja2_native) is V(False).
             - This offers more flexibility than the template module which does not use Jinja2 native types at all.
-            - Mutually exclusive with the convert_data option.
-        default: False
+        default: True
         version_added: '2.11'
         type: bool
+        deprecated:
+          why: This option is no longer used in the Ansible Core code base.
+          version: "2.23"
+          alternatives: Jinja2 native mode is now the default and only option.
       template_vars:
         description: A dictionary, the keys become additional variables available for templating.
         default: {}
@@ -50,10 +56,22 @@ DOCUMENTATION = """
         description: The string marking the beginning of a comment statement.
         version_added: '2.12'
         type: str
+        default: '{#'
       comment_end_string:
         description: The string marking the end of a comment statement.
         version_added: '2.12'
         type: str
+        default: '#}'
+      trim_blocks:
+        description:
+        - Determine when newlines should be removed from blocks.
+        - When set to V(yes) the first newline after a block is removed (block, not variable tag!).
+        type: bool
+        default: yes
+        version_added: '2.19'
+    seealso:
+      - ref: playbook_task_paths
+        description: Search paths used for relative templates.
 """
 
 EXAMPLES = """
@@ -68,6 +86,11 @@ EXAMPLES = """
 - name: show templating results with different comment start and end string
   ansible.builtin.debug:
     msg: "{{ lookup('ansible.builtin.template', './some_template.j2', comment_start_string='[#', comment_end_string='#]') }}"
+
+- name: show templating results with trim_blocks
+  ansible.builtin.debug:
+    msg: "{{ lookup('ansible.builtin.template', './some_template.j2', trim_blocks=True) }}"
+
 """
 
 RETURN = """
@@ -80,14 +103,11 @@ _raw:
 from copy import deepcopy
 import os
 
-import ansible.constants as C
-
 from ansible.errors import AnsibleError
 from ansible.plugins.lookup import LookupBase
-from ansible.module_utils._text import to_text
-from ansible.template import generate_ansible_template_vars, AnsibleEnvironment
+from ansible.template import trust_as_template
+from ansible._internal._templating import _template_vars
 from ansible.utils.display import Display
-from ansible.utils.native_jinja import NativeJinjaText
 
 
 display = Display()
@@ -95,25 +115,21 @@ display = Display()
 
 class LookupModule(LookupBase):
 
-    def run(self, terms, variables, **kwargs):
+    def run(self, terms, variables=None, **kwargs):
 
         ret = []
 
         self.set_options(var_options=variables, direct=kwargs)
 
         # capture options
-        convert_data_p = self.get_option('convert_data')
         lookup_template_vars = self.get_option('template_vars')
-        jinja2_native = self.get_option('jinja2_native') and C.DEFAULT_JINJA2_NATIVE
         variable_start_string = self.get_option('variable_start_string')
         variable_end_string = self.get_option('variable_end_string')
         comment_start_string = self.get_option('comment_start_string')
         comment_end_string = self.get_option('comment_end_string')
+        trim_blocks = self.get_option('trim_blocks')
 
-        if jinja2_native:
-            templar = self._templar
-        else:
-            templar = self._templar.copy_with_new_env(environment_class=AnsibleEnvironment)
+        templar = self._templar
 
         for term in terms:
             display.debug("File lookup term: %s" % term)
@@ -121,8 +137,7 @@ class LookupModule(LookupBase):
             lookupfile = self.find_file_in_search_path(variables, 'templates', term)
             display.vvvv("File lookup using %s as file" % lookupfile)
             if lookupfile:
-                b_template_data, show_data = self._loader._get_file_contents(lookupfile)
-                template_data = to_text(b_template_data, errors='surrogate_or_strict')
+                template_data = trust_as_template(self._loader.get_text_file_contents(lookupfile))
 
                 # set jinja2 internal search path for includes
                 searchpath = variables.get('ansible_search_path', [])
@@ -141,22 +156,25 @@ class LookupModule(LookupBase):
                 # plus some added by ansible (e.g., template_{path,mtime}),
                 # plus anything passed to the lookup with the template_vars=
                 # argument.
+                # FIXME: why isn't this a chainmap with a sacrificial bottom layer?
                 vars = deepcopy(variables)
-                vars.update(generate_ansible_template_vars(term, lookupfile))
+                vars.update(_template_vars.generate_ansible_template_vars(
+                    path=term,
+                    fullpath=lookupfile,
+                    include_ansible_managed='ansible_managed' not in vars,  # do not clobber ansible_managed when set by the user
+                ))
                 vars.update(lookup_template_vars)
 
-                with templar.set_temporary_context(variable_start_string=variable_start_string,
-                                                   variable_end_string=variable_end_string,
-                                                   comment_start_string=comment_start_string,
-                                                   comment_end_string=comment_end_string,
-                                                   available_variables=vars, searchpath=searchpath):
-                    res = templar.template(template_data, preserve_trailing_newlines=True,
-                                           convert_data=convert_data_p, escape_backslashes=False)
+                overrides = dict(
+                    variable_start_string=variable_start_string,
+                    variable_end_string=variable_end_string,
+                    comment_start_string=comment_start_string,
+                    comment_end_string=comment_end_string,
+                    trim_blocks=trim_blocks,
+                )
 
-                if (C.DEFAULT_JINJA2_NATIVE and not jinja2_native) or not convert_data_p:
-                    # jinja2_native is true globally but off for the lookup, we need this text
-                    # not to be processed by literal_eval anywhere in Ansible
-                    res = NativeJinjaText(res)
+                data_templar = templar.copy_with_new_env(available_variables=vars, searchpath=searchpath)
+                res = data_templar.template(template_data, escape_backslashes=False, overrides=overrides)
 
                 ret.append(res)
             else:
