@@ -15,9 +15,7 @@
 # You should have received a copy of the GNU General Public License
 # alongwith Ansible.  If not, see <http://www.gnu.org/licenses/>.
 
-# Make coding more python3-ish
-from __future__ import (absolute_import, division, print_function)
-__metaclass__ = type
+from __future__ import annotations
 
 import shlex
 
